from edge.service.tz import ITzService
from .types import ScheduleArgs


class Schedule:
    def __init__(self, args, tz_service):
        """
        :type args: ScheduleArgs | None
        :type tz_service: ITzService
        """
        self._disabled = args is None
        self._args = args
        self._tz_service = tz_service

    def check(self, sec_ts):
        """
        :type sec_ts: int
        :rtype: bool
        """
        if self._disabled:
            return True

        dt = self._tz_service.get_datetime(sec_ts)

        if not self._args.days_to_bool[dt.weekday()]:
            return False

        if self._args.min_time.minute == self._args.min_time.hour == \
                self._args.max_time.minute == self._args.max_time.hour == 0:
            return True

        if not(self._args.min_time < dt.time() < self._args.max_time):
            return False

        return True

    def check2(self, usec_ts):
        """
        :type usec_ts: int
        :rtype: bool
        """
        return self.check(int(usec_ts / 1e6))
