from ...model.types import Evidence, Identity, Mask
from edge.model.types import Dir


class LimitsArgs:
    def __init__(self, speed_limit, over_speed):
        """
        :type speed_limit: int
        :type over_speed: int
        """
        self.speed_limit = speed_limit
        self.over_speed = over_speed


class RuleArgs:
    def __init__(self, name, enabled, schedule, ticket, limits):
        """
        :type name: str
        :type enabled: bool
        :type schedule: ScheduleArgs
        :type ticket: Ticket
        :type limits: LimitsArgs
        """
        self.name = name
        self.enabled = enabled
        self.schedule = schedule
        self.ticket = ticket
        self.limits = limits


class InitAreaArgs:
    def __init__(self, controller, auth, origin):
        """
        :type controller: str
        :type auth: (str, str)
        :type origin: str
        """
        self.controller = controller
        self.auth = auth
        self.origin = origin


class FiniAreaArgs:
    def __init__(self, mask, scene_wh):
        """
        :type mask: Mask
        :type scene_wh: (int, int)
        """
        self.mask = mask
        self.scene_wh = scene_wh


class SectionArgs:
    def __init__(self, init_area, fini_area, direction, dist):
        """
        :type init_area: InitAreaArgs
        :type fini_area: FiniAreaArgs
        :type direction: Dir
        :type dist: int
        """
        self.init_area = init_area
        self.fini_area = fini_area
        self.direction = direction
        self.dist = dist


class DetectorArgs:
    def __init__(self, identity, name, store, rules, im_engine,
                 def_footer_mapping, section):
        """
        :type identity: Identity
        :type name: str
        :type store: str
        :type rules: list of RuleArgs
        :type im_engine: ImEngineArgs
        :type def_footer_mapping: dict
        :type section: SectionArgs
        """
        self.identity = identity
        self.name = name
        self.store = store
        self.rules = rules
        self.im_engine = im_engine
        self.def_footer_mapping = def_footer_mapping
        self.section = section


class SectionSpeedEvidence(Evidence):
    def __init__(self, origin, ts, section_speed, rule_args, detector_name, direction, detector_dist, measured_dist):
        """
        :type origin: Origin
        :type ts: long
        :type rule_args: RuleArgs
        :type detector_name: str
        :type direction: Dir
        :type detector_dist: int
        :type measured_dist: float
        """
        Evidence.__init__(self, Evidence.NAME_SECTION_SPEED, origin, ts)
        self.detector_name = detector_name
        self.section_speed = section_speed
        self.rule_args = rule_args
        self.dir = direction
        self.detector_dist = detector_dist
        self.measured_dist = measured_dist

    def serialize(self, tz_service):
        """
        :type tz_service: ITzService
        :rtype: dict
        """
        jnode = Evidence.serialize(self, tz_service)
        waypoints_jnode = list()
        for waypoint in self._waypoints:
            waypoints_jnode.append(waypoint.serialize(tz_service))

        jnode['evidence'] = {
            'detector': {
                'name': self.detector_name,
                'dir': str(self.dir),
                'dist': self.detector_dist,
                'rule': {
                    'name': self.rule_args.name,
                    'schedule': self.rule_args.schedule.serialize(),
                    'ticket': {
                        'name': self.rule_args.ticket.name
                    },
                    'limits': {
                        'speed-limit': self.rule_args.limits.speed_limit,
                        'max-overspeed': self.rule_args.limits.over_speed
                    }
                }
            },
            'measurements': {
                'section-dist': self.measured_dist,
                'section-speed': self.section_speed,
                'waypoints': {
                    'init': waypoints_jnode[0],
                    'fini': waypoints_jnode[1]
                }
            }
        }

        return jnode
