from edge import eventlog
from edge.types import JnodeError

from ..types import DetectorArgs, ImagesArgs
from ...types import IDetector
from ...com.jnode.types import JsonScheduleArgs

from ....model.types import Identity


class JsonImagesArgs(ImagesArgs):
    def __init__(self, *args, **kwargs):
        ImagesArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(args_jnode):
        """
        :type args_jnode: dict
        :rtype: JsonImagesArgs
        :raise JnodeError:
        """
        def deserialize_store(im_profile):
            profile_jnode = args_jnode.get(im_profile)
            if dict != type(profile_jnode):
                eventlog.log_e_param('#%s' % im_profile)
                raise JnodeError

            store = profile_jnode.get('store')
            if not (type(store) in [unicode, str] and 1 <= len(store) <= 8):
                eventlog.log_e_param('#/%s/store' % im_profile)
                raise JnodeError

            return store

        scene_store, vehicle_store = map(
            deserialize_store, ['scene', 'vehicle'])

        return JsonImagesArgs(vehicle_store, scene_store)


class JsonDetectorArgs(DetectorArgs):
    def __init__(self, *args, **kwargs):
        DetectorArgs.__init__(self, *args, **kwargs)

    @staticmethod
    def deserialize(transit_jnode):
        """
        :type transit_jnode: dict
        :rtype: JsonDetectorArgs
        """
        rules_jnode = transit_jnode['rules']

        # deserialize #/rules
        if type(rules_jnode) != list or len(rules_jnode) != 1:
            eventlog.log_e_param('#/rules')
            raise JnodeError

        # deserialize #/rules[0]/schedule
        schedule_args = None
        schedule_jnode = rules_jnode[0].get('schedule')
        if schedule_jnode is not None:
            if dict != type(schedule_jnode):
                eventlog.log_e_param('#/schedule')
                raise JnodeError

            schedule_args = JsonScheduleArgs.deserialize(schedule_jnode)

        # deserialize #/rules[0]/name
        name = rules_jnode[0].get('name')
        if type(name) not in [str, unicode]:
            eventlog.log_e_param('#/name')
            raise JnodeError

        # deserialize #/rules[0]/enabled
        enabled = rules_jnode[0].get('enabled')
        if type(enabled) != bool:
            eventlog.log_e_param('#/enabled')
            raise JnodeError

        # deserialize #/rules[0]/limits
        limits_jnode = rules_jnode[0].get('limits')
        if type(limits_jnode) != dict:
            eventlog.log_e_param('#/limits')
            raise JnodeError

        require_plate = limits_jnode.get('require-plate', True)
        if type(require_plate) != bool:
            eventlog.log_e_param('#/limits/require-plate')
            raise JnodeError

        # deserialize #/images
        images_jnode = transit_jnode.get('images')
        if type(images_jnode) != dict:
            eventlog.log_e_param('#/images')
            raise JnodeError

        # deserialize #/images/best
        best_waypoint_jnode = images_jnode.get('best')
        if type(best_waypoint_jnode) != dict:
            eventlog.log_e_param('#/images/best')
            raise JnodeError

        images = JsonImagesArgs.deserialize(best_waypoint_jnode)

        identity = Identity(IDetector.TYPE_TRANSIT, 0)
        plate_detected_mode = transit_jnode.get('plate-detected-mode', False)
        return JsonDetectorArgs(name, enabled, require_plate, schedule_args, images, identity, plate_detected_mode)
