from edge.image.types import MemBlob
from edge.service.tz import ITzService
from edge.model.types import Origin

from ...model.types import Identity, Evidence
from ..com.types import ScheduleArgs


class ImagesArgs:
    def __init__(self, vehicle_store, scene_store):
        """
        :type vehicle_store: str
        :type scene_store: str
        """
        self.vehicle_store = vehicle_store
        self.scene_store = scene_store


class DetectorArgs:
    def __init__(self, name, enabled, require_plate, schedule_args, blob_store_args, identity, plate_detected_mode=False):
        """
        :type name: str
        :type enabled: bool
        :type require_plate: bool
        :type schedule_args: ScheduleArgs | None
        :type blob_store_args: ImagesArgs
        :type identity: Identity
        :type plate_detected_mode: bool
        """
        self.name = name
        self.enabled = enabled
        self.require_plate = require_plate
        self.schedule_args = schedule_args
        self.blob_store_args = blob_store_args
        self.identity = identity
        self.plate_detected_mode = plate_detected_mode


class TransitEvidence(Evidence):

    def __init__(self, origin, ts):
        """
        :type origin: Origin
        :type ts: long
        """
        Evidence.__init__(self, Evidence.NAME_TRANSIT, origin, ts)

    def get_blobs(self):
        """
        :rtype: list of MemBlob
        """
        return []

    def serialize(self, tz_service):
        """
        :type tz_service: ITzService
        :rtype: dict
        """
        jnode = Evidence.serialize(self, tz_service)
        return jnode
